package com.jiuqi.sql.impl;

import com.jiuqi.util.JqLib;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Calendar;
import java.util.Date;

/* loaded from: com_jiuqi_sql_impl.jar:com/jiuqi/sql/impl/RW.class */
public class RW {
    private static final int dayOfYear = 365;
    private static final int dayOf4Year = 1461;
    private static final int dayOfCentury = 36524;
    private static final int dayOf4Century = 146097;
    private static final int[][] monthDays = {new int[]{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}, new int[]{31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}};
    private static final int hoursPerDay = 24;
    private static final int minsPerHour = 60;
    private static final int secsPerMin = 60;
    private static final int mSecsPerSec = 1000;
    private static final int minsPerDay = 1440;
    private static final int secsPerDay = 86400;
    private static final int mSecsPerDay = 86400000;

    private static int[] decodeDelphiDateFully(double dateTime) {
        int[] result = new int[5];
        int t = ((int) dateTime) + 693594;
        if (t > 0) {
            result[3] = (t % 7) + 1;
            int t2 = t - 1;
            int y = 1;
            while (t2 >= dayOf4Century) {
                t2 -= dayOf4Century;
                y += 400;
            }
            int i = t2 / dayOfCentury;
            int d = t2 % dayOfCentury;
            if (i == 4) {
                i--;
                d += dayOfCentury;
            }
            int i2 = d / dayOf4Year;
            int d2 = d % dayOf4Year;
            int y2 = y + (i * 100) + (i2 * 4);
            int i3 = d2 / dayOfYear;
            int d3 = d2 % dayOfYear;
            if (i3 == 4) {
                i3--;
                d3 += dayOfYear;
            }
            int y3 = y2 + i3;
            result[4] = (y3 % 4 != 0 || (y3 % 100 == 0 && y3 % 400 != 0)) ? 0 : 1;
            int[] dayTable = monthDays[result[4]];
            int m = 1;
            while (true) {
                int i4 = dayTable[m - 1];
                if (d3 < i4) {
                    break;
                }
                d3 -= i4;
                m++;
            }
            result[0] = y3;
            result[1] = m - 1;
            result[2] = d3 + 1;
        }
        return result;
    }

    private static double encodeDelphiDateFully(int year, int dayOfYear2) {
        int dayCount = ((year - 1) - 1899) * dayOfYear;
        int leapYearCount = 0;
        for (int i = 1900; i <= year - 1; i++) {
            if (JqLib.isLeapYear(i)) {
                leapYearCount++;
            }
        }
        return dayCount + leapYearCount + 1 + dayOfYear2;
    }

    private static int[] decodeDelphiTime(double dateTime) {
        int time = (int) (8.64E7d * (dateTime - ((int) dateTime)));
        int minCount = time / 60000;
        int mSecCount = time % 60000;
        int[] result = {minCount / 60, minCount % 60, mSecCount / mSecsPerSec, mSecCount % mSecsPerSec};
        return result;
    }

    private static double encodeDelphiTime(int hour, int minute, int second, int msec) {
        int minCount = (60 * hour) + minute;
        int mSecCount = (mSecsPerSec * second) + msec;
        int time = (60000 * minCount) + mSecCount;
        return time / 8.64E7d;
    }

    private static Date decodeDelphiDateTime(double dateTime) {
        int[] date = decodeDelphiDateFully(dateTime);
        int[] time = decodeDelphiTime(dateTime);
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(0L);
        calendar.set(date[0], date[1], date[2], time[0], time[1], time[2]);
        return calendar.getTime();
    }

    private static double encodeDelphiDateTime(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(0L);
        calendar.setTime(date);
        int year = calendar.get(1);
        int dayOfYear2 = calendar.get(6);
        int hour = calendar.get(10);
        int minute = calendar.get(12);
        int sec = calendar.get(13);
        int mSec = calendar.get(14);
        return encodeDelphiDateFully(year, dayOfYear2) + encodeDelphiTime(hour, minute, sec, mSec);
    }

    public static byte read(InputStream stream) throws IOException {
        return (byte) stream.read();
    }

    public static void write(OutputStream stream, byte value) throws IOException {
        stream.write(value);
    }

    public static boolean readBoolean(InputStream stream) throws IOException {
        return stream.read() > 0;
    }

    public static void writeBoolean(OutputStream stream, boolean value) throws IOException {
        stream.write(value ? 1 : 0);
    }

    public static boolean readBoolean(byte[] data, int index) {
        return data[index] == 84 || data[index] == 116 || data[index] == 49;
    }

    public static void writeBoolean(byte[] data, int index, boolean value) {
        if (value) {
            data[index] = 84;
        } else {
            data[index] = 70;
        }
    }

    public static String readString(InputStream stream, int len) throws IOException {
        byte[] str = new byte[len];
        stream.read(str, 0, len);
        return bytesToString(str);
    }

    public static void writeString(OutputStream stream, String value, int len) throws IOException {
        int l = 0;
        if (value != null) {
            byte[] str = value.getBytes();
            stream.write(str, 0, str.length > len ? len : str.length);
            if (str.length > len) {
                return;
            } else {
                l = str.length;
            }
        }
        for (int i = 1; i <= len - l; i++) {
            stream.write(0);
        }
    }

    private static String bytesToString(byte[] data) {
        if (data == null || data.length == 0) {
            return null;
        }
        for (int i = data.length - 1; i >= 0; i--) {
            if (data[i] != 0 && data[i] != 32) {
                return new String(data, 0, i + 1);
            }
        }
        return "";
    }

    public static String readString(byte[] data, int index, int len) {
        byte[] str = new byte[len];
        System.arraycopy(data, index, str, 0, len);
        return bytesToString(str);
    }

    public static void writeString(byte[] data, int index, String value, int len) {
        byte[] dataToWrite = value.getBytes();
        System.arraycopy(dataToWrite, 0, data, index, dataToWrite.length > len ? len : dataToWrite.length);
        if (len > dataToWrite.length) {
            byte[] restData = new byte[len - dataToWrite.length];
            int ind = dataToWrite.length + index;
            System.arraycopy(restData, 0, data, ind, restData.length);
        }
    }

    public static Date readDateTime(byte[] data, int index) {
        double datetime = readDouble(data, index);
        return decodeDelphiDateTime(datetime);
    }

    public static Date readDateTime(InputStream stream) throws IOException {
        double datetime = readDouble(stream);
        return decodeDelphiDateTime(datetime);
    }

    public static void writeDateTime(byte[] data, int index, Date dateValue) {
        double datetime = encodeDelphiDateTime(dateValue);
        writeDouble(data, index, datetime);
    }

    public static void writeDateTime(OutputStream stream, Date dateValue) throws IOException {
        double datetime = encodeDelphiDateTime(dateValue);
        writeDouble(stream, datetime);
    }

    public static void writeDateTime(OutputStream stream, long dateValue) throws IOException {
        writeLong(stream, dateValue);
    }

    public static void writeDateTime(byte[] data, int index, long dateValue) {
        writeLong(data, index, dateValue);
    }

    public static double readDouble(InputStream stream) throws IOException {
        return Double.longBitsToDouble(readLong(stream));
    }

    public static void writeDouble(OutputStream stream, double value) throws IOException {
        writeLong(stream, Double.doubleToLongBits(value));
    }

    public static double readDouble(byte[] data, int index) {
        return Double.longBitsToDouble(readLong(data, index));
    }

    public static void writeDouble(byte[] data, int index, double value) {
        writeLong(data, index, Double.doubleToLongBits(value));
    }

    public static float readFloat(InputStream stream) throws IOException {
        return Float.intBitsToFloat(readInt(stream));
    }

    public static void writeFloat(OutputStream stream, float value) throws IOException {
        writeInt(stream, Float.floatToIntBits(value));
    }

    public static float readFloat(byte[] data, int index) {
        return Float.intBitsToFloat(readInt(data, index));
    }

    public static void writeFloat(byte[] data, int index, float value) {
        writeInt(data, index, Float.floatToIntBits(value));
    }

    public static int readInt(InputStream stream) throws IOException {
        int b1 = stream.read() & 255;
        int b2 = stream.read() & 255;
        int b3 = stream.read() & 255;
        int b4 = stream.read() & 255;
        int result = (b4 << 0) + (b3 << 8) + (b2 << 16) + (b1 << hoursPerDay);
        return result ^ Integer.MIN_VALUE;
    }

    public static void writeInt(OutputStream stream, int value) throws IOException {
        int value2 = value ^ Integer.MIN_VALUE;
        byte[] b = {(byte) ((value2 >> hoursPerDay) & 255), (byte) ((value2 >> 16) & 255), (byte) ((value2 >> 8) & 255), (byte) (value2 >> 0)};
        stream.write(b);
    }

    public static int readInt(byte[] data, int index) {
        int b1 = data[index] & 255;
        int b2 = data[index + 1] & 255;
        int b3 = data[index + 2] & 255;
        int b4 = data[index + 3] & 255;
        int result = (b4 << 0) + (b3 << 8) + (b2 << 16) + (b1 << hoursPerDay);
        return result ^ Integer.MIN_VALUE;
    }

    public static void writeInt(byte[] data, int index, int value) {
        int value2 = value ^ Integer.MIN_VALUE;
        data[index + 3] = (byte) (value2 >> 0);
        data[index + 2] = (byte) ((value2 >> 8) & 255);
        data[index + 1] = (byte) ((value2 >> 16) & 255);
        data[index + 0] = (byte) ((value2 >> hoursPerDay) & 255);
    }

    public static long readLong(InputStream stream) throws IOException {
        long b1 = stream.read() & 255;
        long b2 = stream.read() & 255;
        long b3 = stream.read() & 255;
        long b4 = stream.read() & 255;
        long b5 = stream.read() & 255;
        long b6 = stream.read() & 255;
        long b7 = stream.read() & 255;
        long b8 = stream.read();
        return (b8 << 56) + (b7 << 48) + (b6 << 40) + (b5 << 32) + (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
    }

    public static void writeLong(OutputStream stream, long value) throws IOException {
        byte[] b = {(byte) (value >> 0), (byte) ((value >> 8) & 255), (byte) ((value >> 16) & 255), (byte) ((value >> 24) & 255), (byte) ((value >> 32) & 255), (byte) ((value >> 40) & 255), (byte) ((value >> 48) & 255), (byte) ((value >> 56) & 255)};
        stream.write(b);
    }

    public static long readLong(byte[] data, int index) {
        long b1 = data[index] & 255;
        long b2 = data[index + 1] & 255;
        long b3 = data[index + 2] & 255;
        long b4 = data[index + 3] & 255;
        long b5 = data[index + 4] & 255;
        long b6 = data[index + 5] & 255;
        long b7 = data[index + 6] & 255;
        long b8 = data[index + 7];
        return (b8 << 56) + (b7 << 48) + (b6 << 40) + (b5 << 32) + (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
    }

    public static void writeLong(byte[] data, int index, long value) {
        data[index] = (byte) (value >> 0);
        data[index + 1] = (byte) ((value >> 8) & 255);
        data[index + 2] = (byte) ((value >> 16) & 255);
        data[index + 3] = (byte) ((value >> 24) & 255);
        data[index + 4] = (byte) ((value >> 32) & 255);
        data[index + 5] = (byte) ((value >> 40) & 255);
        data[index + 6] = (byte) ((value >> 48) & 255);
        data[index + 7] = (byte) ((value >> 56) & 255);
    }

    public static int readWord(InputStream stream) throws IOException {
        int b1 = stream.read() & 255;
        int b2 = stream.read() & 255;
        return (b2 << 8) + (b1 << 0);
    }

    public static void writeWord(OutputStream stream, int value) throws IOException {
        byte[] b = {(byte) value, (byte) ((value >> 8) & 255)};
        stream.write(b);
    }

    public static int readWord(byte[] data, int index) {
        int b1 = data[index] & 255;
        int b2 = data[index + 1] & 255;
        return (b2 << 8) + (b1 << 0);
    }

    public static void writeWord(byte[] data, int index, int value) {
        data[index] = (byte) (value >> 0);
        data[index + 1] = (byte) ((value >> 8) & 255);
    }

    public static long readDWord(InputStream stream) throws IOException {
        int b1 = stream.read() & 255;
        int b2 = stream.read() & 255;
        int b3 = stream.read() & 255;
        int b4 = stream.read() & 255;
        return (b4 << hoursPerDay) + (b3 << 16) + (b2 << 8) + (b1 << 0);
    }

    public static void writeDWord(OutputStream stream, long value) throws IOException {
        byte[] b = {(byte) (value >> 0), (byte) ((value >> 8) & 255), (byte) ((value >> 16) & 255), (byte) ((value >> 24) & 255)};
        stream.write(b);
    }

    public static long readDWord(byte[] data, int index) {
        int b1 = data[index] & 255;
        int b2 = data[index + 1] & 255;
        int b3 = data[index + 2] & 255;
        int b4 = data[index + 3] & 255;
        return (b4 << hoursPerDay) + (b3 << 16) + (b2 << 8) + (b1 << 0);
    }

    public static void writeDWord(byte[] data, int index, long value) {
        data[index] = (byte) (value >> 0);
        data[index + 1] = (byte) ((value >> 8) & 255);
        data[index + 2] = (byte) ((value >> 16) & 255);
        data[index + 3] = (byte) ((value >> 24) & 255);
    }
}
